/*********************************************************************
 * Desktop Station AUTO Sketch for DS shield series
 *
 * Copyright (C) 2017 Yaasan
 *
 */

// include the library code:
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "Functions.h"
#include "TrackReporterS88_DS.h"

#define MAX_S88DECODER 1

#define LEDSTATE_STOP 0
#define LEDSTATE_RUN 1
#define LEDSTATE_ERR 2

#define PIN_RUNLED	2

#define MODE_SPEEDSTEP128 2

#define THRESHOLD_EDC 276  // 1024d * (1.2 / 8) * 9V / 5V
#define THRESHOLD_OV  614  // 1024d * (1.2 / 8) * 20V / 5V


/* Decralation classes */
DSCoreLib DSCore;
TrackReporterS88_DS reporter(MAX_S88DECODER);

/* LED functions */
byte gLED_State = LEDSTATE_STOP;
byte gLED_Toggle = 0;

//Task Schedule
unsigned long gPreviousL8 = 0; // 1000ms interval


//コマンド関数定義
void CMD_Wait(word inWaitMs);
void CMD_Power(byte inState);
void CMD_LocSpeed(word inAddress, word inSpeed);
void CMD_LocDirection(word inAddress, byte inDirection);
void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState);
void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo);
void CMD_Turnout(word inAddress, byte inDirection);
void CMD_ScanS88(void);
byte CMD_GetS88(byte inNo);
void CMD_END(void);

/***********************************************************************
* 
* Boot setup
* 
************************************************************************/

void setup()
{
	DSCore.Init();

	//Pin mode
	pinMode(A1, INPUT);//Edc
	
	/* LED turn on */
	changeLED(0);
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* Debug message for PC */
	Serial.begin(115200);
	Serial.println("--------------------------------------");
	Serial.println("Desktop Station AUTO R1 2017/11/12  ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");
	
	//Reset task
	gPreviousL8 = millis();

	//Timer1 init
	TCCR1A = 0;
	TCCR1B = 0;
	TCNT1  = 0;

	OCR1A = 20;            // 1Hz = 62500d, 100Hz = 625d
	TCCR1B |= (1 << WGM12);   // CTC mode
	TCCR1B |= (1 << CS12);    // 256 prescaler 
	TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
	
	sei();    //ENABLE INTERRUPTION
	
	//User initial
	user_init();
	
}

/***********************************************************************
* 
* Command for locomotives and turnouts
* 
************************************************************************/

void CMD_Wait(word inWaitMs)
{
	unsigned long aTemp = millis() + (inWaitMs >> 2);
	
	//なんか時間が合わない。Timer0のプリスケーラがおかしい？
	while ( aTemp > millis() )
	{
		//NOP
		__asm__("nop\n\t");
	}
	
}

void CMD_Power(byte inState)
{
	/* LED状態の変更 */
	changeLED(inState);
	
	DSCore.SetPower(inState);
}

void CMD_LocSpeed(word inAddress, word inSpeed)
{
	DSCore.SetLocoSpeedEx(inAddress, inSpeed, MODE_SPEEDSTEP128);
}

void CMD_LocDirection(word inAddress, byte inDirection)
{
	DSCore.SetLocoDirection(inAddress, inDirection);
}

void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, inFunctionState);
}

void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 1);
	CMD_Wait(10);
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 0);
}


void CMD_Turnout(word inAddress, byte inDirection)
{
	DSCore.SetTurnout(inAddress - 1, inDirection);
}

void CMD_ScanS88(void)
{
	reporter.refresh(MAX_S88DECODER);
}

byte CMD_GetS88(byte inNo)
{
	word aFlags = reporter.getByte(0) + reporter.getByte(1);
	
	return (aFlags >> inNo) & 1;
}

void CMD_END(void)
{
	/* 永久ループ（終了） */
	while(1){}
}


/***********************************************************************
* 
* Timer Interval
* 
************************************************************************/

ISR(TIMER1_COMPA_vect)
{
	uint8_t aErrNo = 0;
	word aEdc;
	aErrNo = 0;
	
	//パルスジェネレータのスキャン
	DSCore.Scan();
	
	if( (millis() - gPreviousL8) >= 200)
	{
		//Reset task
		gPreviousL8 = millis();
		
		//LED Control
		taskLED();
		
		/* Sensor check */
		aEdc = analogRead(A1);
		
		/* OV check */
		if( aEdc > THRESHOLD_OV )
		{
			aErrNo = 2;
		}
		/* LV check */
		if( aEdc < THRESHOLD_EDC )
		{
			aErrNo = 1;
			
		}
		/* エラー処理 */
		if( aErrNo > 0)
		{
			if( DSCore.IsPower() == 1)
			{
				PowerOffByErr(aErrNo);
			}
			
		}
	}
	
}

/***********************************************************************
* 
* Main Loop
* 
************************************************************************/

void loop()
{
	
	user_program();
	
}

/***********************************************************************
* 
* RUN LED Control functions
* 
************************************************************************/

void PowerOffByErr(uint8_t inErrNo)
{
	DSCore.SetPower(0);
	
	/* 状態変更 */
	gLED_State = LEDSTATE_ERR;
}


void changeLED(byte inState)
{
	if( inState == 1)
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_RUN;
	}
	else
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_STOP;
	}
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		digitalWrite(PIN_RUNLED, 0);
		
		break;
		
	case LEDSTATE_RUN:
		digitalWrite(PIN_RUNLED, 1);
		break;
		
	case LEDSTATE_ERR:
		break;
	}
}

void taskLED(void)
{
	
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		break;
		
	case LEDSTATE_RUN:
		break;
		
	case LEDSTATE_ERR:
		digitalWrite(PIN_RUNLED, gLED_Toggle);
		gLED_Toggle = (~gLED_Toggle) & 1;
		
		break;
	}
}

/*****************************************************************/
/*  User Program Area                                            */
/*****************************************************************/


//Task Schedule
unsigned long gPreviousL1 = 0;
unsigned long gPreviousL2 = 0;

int s88_a,s88_b;
const int loco_num = 3 + ADDR_DCC; 
boolean s88_get_flag_a,s88_get_flag_b;
const int loco_speed = 1000;

void user_init(void)
{
	
	pinMode(8, INPUT);
	pinMode(11, INPUT);
	pinMode(12, INPUT);
	
	digitalWrite(8, HIGH);//内部プルアップON
	digitalWrite(11, HIGH);//内部プルアップON
	digitalWrite(12, HIGH);//内部プルアップON
	
	CMD_Power(1);
	Serial.println("Power On");
  delay(500);
  //light on
  CMD_LocFunction(loco_num,0,1);
  s88_get_flag_a = true;
  s88_get_flag_b = false;
  
  //Reset task
  gPreviousL1 = millis();
  gPreviousL2 = millis();
  
}


void user_program(void)
{
  //S88の状態を返す
  if( (millis() - gPreviousL1) >= 200)
  {
    CMD_ScanS88();
    gPreviousL1 = millis();
  }

  if( (millis() - gPreviousL2) >= 200)
  {
    //S88 Get
    if(s88_get_flag_a == true)
    {
      s88_a = CMD_GetS88(0);
      Serial.print("s88_a,");
      Serial.println(s88_a);
    }
    if(s88_get_flag_b == true)
    {
      s88_b = CMD_GetS88(1);
      Serial.print("s88_b,");
      Serial.println(s88_b);
    }
    //Reset task
    gPreviousL2 = millis();
  }

  //event_a
  if(s88_a == 1)
  {
    Serial.print("event_a,start");
    s88_get_flag_a = false;
    s88_get_flag_b = false;
    s88_a = 0;
    s88_b = 0;
    //stop
    CMD_LocSpeed(loco_num,0);
    delay(1000);
    //stop engine
    CMD_LocFunction(loco_num,3,0);
    delay(500);
    //方向転換
    CMD_LocDirection(loco_num,0);
    delay(300);
    //start engine
    CMD_LocFunction(loco_num,3,1);
    delay(1000);
    CMD_LocFunction(loco_num,8,1);
    delay(1000);
    //start
    CMD_LocSpeed(loco_num,loco_speed);
    s88_get_flag_a = false;
    s88_get_flag_b = true;
    Serial.print("event_a,end");
  }

  //event_b
  if(s88_b == 1)
  {
    Serial.print("event_b,start");
    s88_get_flag_a = false;
    s88_get_flag_b = false;
    s88_a = 0;
    s88_b = 0;
    //stop
    CMD_LocSpeed(loco_num,0);
    delay(500);
    //方向転換
    CMD_LocDirection(loco_num,1);
    delay(1000);
    CMD_LocFunction(loco_num,8,0);
    delay(1000);
    //start
    CMD_LocSpeed(loco_num,loco_speed);
    s88_get_flag_a = true;
    s88_get_flag_b = false;
    Serial.print("event_b,end");
  }
    
}



